from importlib import import_module
from functools import partial, update_wrapper


class Router:
    '''
    路由器类
    '''

    def __init__(self, module):
        '''
        初始化。
        '''

        self.module = module
        self.actions = []

    def route(self, app):
        '''
        添加路由到应用。
        '''

        for cn, n, uri, args, kwargs in self.actions:
            if cn != None:
                controller_class = getattr(self.module, cn)
                controller = controller_class()
                method = getattr(controller_class, n)
                handler = partial(method, controller)
                update_wrapper(handler, method)
                app.add_route(handler, uri, *args, **kwargs)
            else:
                handler = getattr(self.module, n)
                app.add_route(handler, uri, *args, **kwargs)

    def append(
        self,
        class_name,  # 类名
        method_name,  # 方法名
        uri,  # 路由 URI
        args,  # 路由 数组参数
        kwargs  # 路由 键值参数
    ):
        '''
        存储路由参数。
        '''

        self.actions.append((
            class_name,
            method_name,
            uri,
            args,
            kwargs
        ))


def action(uri,  *args, **kwargs):
    '''
    路由信息设置。
    '''

    def decorator(method):
        '''
        装饰器。
        '''

        n = method.__name__
        qn = method.__qualname__
        cn = qn[:-len(n) - 1] if qn != n else None
        mn = method.__module__
        m = import_module(mn)
        if not hasattr(m, '___router___'):
            setattr(m, '___router___', Router(m))
        m.___router___.append(cn, n, uri, args, kwargs)
        return method
    return decorator


def any(uri, *args, **kwargs):
    '''
    接受所有方法类型。
    '''

    kwargs['methods'] = [
        'HEAD',
        'GET',
        'POST',
        'PUT',
        'DELETE',
        'CONNECT',
        'OPTIONS',
        'TRACE',
        'PATCH',
    ]
    return action(uri, *args, **kwargs)


def get(uri,  *args, **kwargs):
    '''
    接受 GET 请求。
    '''

    kwargs['methods'] = ['GET']
    return action(uri, *args, **kwargs)


def put(uri,  *args, **kwargs):
    '''
    接受 PUT 请求。
    '''

    kwargs['methods'] = ['PUT']
    return action(uri, *args, **kwargs)


def delete(uri,  *args, **kwargs):
    '''
    接受 DELETE 请求。
    '''

    kwargs['methods'] = ['DELETE']
    return action(uri, *args, **kwargs)


def post(uri,  *args, **kwargs):
    '''
    接受 POST 请求。
    '''

    kwargs['methods'] = ['POST']
    return action(uri, *args, **kwargs)



from pkgutil import iter_modules
from importlib import import_module
from sanic import Sanic


class Application(Sanic):
    '''
    应用类
    '''

    def __init__(
        self,
        name=None,  # 应用名
        router=None,  # 路由
        error_handler=None,  # 错误处理
        load_env=True,
        request_class=None,
        strict_slashes=False,
        log_config=None,  # 日志配置
        configure_logging=True
    ):
        '''
        初始化。
        '''

        super().__init__(
            name=name,
            router=router,
            error_handler=error_handler,
            load_env=load_env,
            request_class=request_class,
            strict_slashes=strict_slashes,
            log_config=log_config,
            configure_logging=configure_logging
        )

    def control(self, module, deep=True):
        '''
        加载控制器。
        '''

        # 加载路由。
        if hasattr(module, '___router___'):
            router = getattr(module, '___router___')
            router.route(self)

        # 是否深入解析。
        name = module.__name__
        if deep and module.__loader__.is_package(name):
            for _, child, _ in iter_modules(module.__path__):
                mn = '{}.{}'.format(name, child)
                m = import_module(mn)
                self.control(m, deep)



from pkgutil import iter_modules
from importlib import import_module


def collect(module, deep=True):
    '''
    获取指定包及子模块名。
    '''

    name = module.__name__
    result = set([name])
    if deep and module.__loader__.is_package(name):
        for _, child, _ in iter_modules(module.__path__):
            mn = '{}.{}'.format(name, child)
            m = import_module(mn)
            result |= collect(m, deep)
    return result


async def initialize(setting, *modules):
    '''
    初始化。
    '''

    models = set()
    for m in modules:
        models |= collect(m)
    setting['modules']['models'] = models



from unittest import TestCase, skip
from test.mock import controller

class RouteTest(TestCase):
    '''
    路由测试。
    '''

    def test_controller(self):
        '''
        测试控制器。
        '''

        tc = controller.TestController()
        r = tc.index()
        self.assertEqual(r, 'index')
        


import setuptools


def load_content(path):
    '''
    加载文件内容。
    '''

    with open(path, 'r', encoding='utf-8') as reader:
        return reader.read()


setuptools.setup(
    name='suoran',
    version='0.0.3',
    description='extends sanic',
    long_description=load_content('readme.md'),
    long_description_content_type='text/markdown',
    url='https://github.com/chenshenchao/suoran',
    keywords='sanic suoran',
    license='MIT',
    author='chenshenchao',
    author_email='chenshenchao@outlook.com',
    platforms='any',
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
    packages=setuptools.find_packages(
        exclude=['test']
    ),
    install_requires=[
        'sanic>=20.9.0',
        'tortoise-orm>=0.16.16',
        'aiomysql>=0.0.20',
    ]
)
